//=================================================================================================
//  RT ݡͥ³饹
//
//  date   : 20080720 first
//  date   : 20080721 update
//
//  Author : Takashi KOJIMA, Chuo University
//
//=================================================================================================
#ifndef INTERFACE_CONNECT_RT_COMPONENT_CLASS
#define INTERFACE_CONNECT_RT_COMPONENT_CLASS

#include "RTCompControl.h"

//!RT component port
/*
*RT port 򽸤᤿¤
*/
struct RTCompSet
{
	RTC::CorbaConsumer<RTC::RTObject> con;       //!< corba comsumer(component)
	int                               numOfPort; //!< select port numbers
  RTC::PortList_var                 pList;     //!< port list
	
	//!ǥեȥ󥹥ȥ饯
	RTCompSet():con(),numOfPort(-1),pList(){};
	//!ԡ󥹥ȥ饯
	RTCompSet(const RTCompSet& src):con(src.con),numOfPort(src.numOfPort),pList(src.pList){};
	//!ǥȥ饯
	~RTCompSet(){};
	//!operator =
	RTCompSet& operator=(const RTCompSet& right)
	{
		con       = right.con;
		numOfPort = right.numOfPort;
		pList     = right.pList;
		return *this;
	};
};

//!RT component connection class
/**
*ڻȤ<BR>
* STEP0 Port䤤碌͡ॵФꤹ SetNameServer()<BR>
* STEP1 PortԤ SetPort()<BR>
* STEP2 ³ Connect()<BR>
*ڻѾա<BR>
* Ĥ³ˤĤΥ饹ϰɬפǤʣ³ˤʣѰդ뤳ȡ<BR>
* ʣˤϡstd::vector<RTCompConnect> std::list<RTCompConnect>ʤɤˤ
* 뤳ȤǽǤ
*/
class RTCompConnect : public ORBvarManager
{
	public:
	//!󥹥ȥ饯
	RTCompConnect(
		 std::string CorbaServeName   //! corba name server
		,long        CorbaPort=-1     //! corba port setting
	);

	//!ԡ󥹥ȥ饯
	RTCompConnect(
		 const RTCompConnect& src     //! source
	);
	
	//!ǥȥ饯
	virtual ~RTCompConnect();

	//!regist both nameserver
	/**
 	* @param ServerName Corba name server's name(Corba name server's ip address)
	* @param OmniPort Corba name server's port number (Example : 9876
	*/
	void SetNameServer(std::string ServerName ,long OmniPort=-1);

	//!regist nameserver
	/**
	* @param num port number[0,1]
 	* @param ServerName Corba name server's name(Corba name server's ip address)
	* @param OmniPort Corba name server's port number (Example : 9876
	*/
	void SetNameServer(const int num, std::string ServerName, long OmniPort=-1);

  //!³Port
	/**
	* @param num port number[0,1]
 	* @param ComponentName Component name (Example : ConsoleIn0.rtc
	* @param PortName Component port name (Example : out
	*/
	bool SetPort( const int num, std::string ComponentName, std::string PortName	);

  //!³Port
	/**
	* @param num port number[0,1]
 	* @param ComponentName Component name (Example : ConsoleIn0.rtc
	* @param PortNumber Component port name (Example : 0
	*/
	bool SetPort( const int num,std::string ComponentName,int PortNumber);

  //!³Port
	/**
	* @param num port number[0,1]
 	* @param plist Component port list (Example : this->get_port_list()
	* @param PortName Component port name (Example : out
	*/
	bool SetPort(const int num, RTC::PortList_var &plist, std::string PortName);

  //!³Port
	/**
	* @param num port number[0,1]
 	* @param plist Component port list (Example : this->get_port_list()
	* @param PortNumber Component port name (Example : 0
	*/
	bool SetPort(const int num, RTC::PortList_var &plist,int PortNumber);

	//!Port³
	/**
	* @param ConnectionName Connection name
	* @param subs_type Connection type (Example : Flush, New,,,)
	* @param period Communication period (Example : 1.0 )
	*/
	bool Connect( std::string ConnectionName, std::string subs_type="Flush",std::string period   ="" );

	//!dosconnect
	inline bool DisConnect(
		std::string ConnectionName      //! Connection name
	);

	//!operator=
	RTCompConnect& operator=(
		const RTCompConnect& right      //! source
	);

	private:
	RTC::CorbaNaming       m_nameserver0;               //!<CorbaNameServer
	RTC::CorbaNaming       m_nameserver1;               //!<CorbaNameServer
	RTCompSet              m_port[2];                   //!<Port infomation
	std::string            m_connect_id;                //!<Connector ID

	protected:
	//!Name server slection function
	/**
	* @param num port number
	*/
	RTC::CorbaNaming* GetNameServer(const int num);

	//!Get port list from nameserver
	/**
	* @param port port parameters variable pointer
	* @param ComponentName RT component name
	* @param NameServer corba name server's pointer
	*/
	void SetPortList  (
		 RTCompSet *port
		,std::string &ComponentName
		,RTC::CorbaNaming *NameServer
	);

	//!Set port numbering from port name
	/**
	* @param port port parameters variable pointer
	* @param PortName port name
	*/
	bool SetPortName  (RTCompSet *port, std::string &PortName);

	//!Set port numbering
	/**
	* @param port port parameters variable pointer
	* @param PortNumber port number
	*/
	bool SetPortNumber(RTCompSet *port, int &PortNumber);

	//!make string from port name and port number
	/**
	* @return corba port
	* @param name corba name server ip address
	* @param port corba name server port
	*/
	std::string NameAndPort2str(std::string name, long port)
	{
		if(port!=-1)
		{
			char PortStr[6];
			sprintf(PortStr,":%ld",port);
			name += PortStr;
		}
		return name;
	};

};


///////////////////////////////////// ؿ /////////////////////////////////

RTCompConnect::RTCompConnect(
		 std::string CorbaServerName   //!name server and
		,long        CorbaPort        //! port setting
)
:m_nameserver0(s_orb,NameAndPort2str(CorbaServerName, CorbaPort).c_str())
,m_nameserver1(s_orb,NameAndPort2str(CorbaServerName, CorbaPort).c_str())
,m_connect_id("")
{
	SetNameServer(CorbaServerName,CorbaPort);
}

RTCompConnect::RTCompConnect
(
	const RTCompConnect& src
):m_nameserver0(src.m_nameserver0)
,m_nameserver1(src.m_nameserver1)
,m_connect_id(src.m_connect_id)
{
	m_port[0] = src.m_port[0];
	m_port[1] = src.m_port[1];
}

RTCompConnect::~RTCompConnect()
{

}

void RTCompConnect::SetNameServer(std::string ServerName, long OmniPort)
{
	ServerName = NameAndPort2str(ServerName, OmniPort);
	try{
		m_nameserver0.init(ServerName.c_str());
		m_nameserver1.init(ServerName.c_str());
	}
	catch(...)
	{
		std::cout << "Error happen. Check what Corbaname server is running." << std::endl;
		exit(1);
	}
}
void RTCompConnect::SetNameServer(const int num, std::string ServerName, long OmniPort)
{
	ServerName = NameAndPort2str(ServerName, OmniPort);
	try{
		GetNameServer(num)->init(ServerName.c_str());
	}
	catch(...)
	{
		std::cout << "Error happen. Check what Corbaname server is running." << std::endl;
		exit(1);
	}
}

RTC::CorbaNaming* RTCompConnect::GetNameServer(const int num)
{
	switch(num)
	{
		case 0:
		return &m_nameserver0;

		case 1:
		return &m_nameserver1;

		default:
		std::cout << "Don't exist number" << std::endl;
	}
	return 0;
}

//push_back_port
bool RTCompConnect::SetPort(const int num, std::string ComponentName,  std::string PortName)
{
	if(num != 0 && num != 1)return false;
	//!STEP1 get port list
	SetPortList(&m_port[num],	ComponentName, GetNameServer(num));
	//!STEP2 identyfy port by name
	return SetPortName(&m_port[num],PortName);

}
//push_back_port
bool RTCompConnect::SetPort(const int num, std::string ComponentName,  int PortNumber)
{
	if(num != 0 && num != 1)return false;
	//!STEP1 get port list
	SetPortList(&m_port[num],	ComponentName, GetNameServer(num));
	//!STEP2 identyfy port by number
	return SetPortNumber(&m_port[num],PortNumber);
}
//push_back_port
bool RTCompConnect::SetPort(const int num, RTC::PortList_var &plist,  std::string PortName)
{
	if(num != 0 && num != 1)return false;
	//!STEP1 set port list
	m_port[num].pList = plist;
	//!STEP2 identyfy port by name
	return SetPortName(&m_port[num],PortName);
}

//push_back_port
bool RTCompConnect::SetPort(const int num, RTC::PortList_var &plist,  int PortNumber)
{
	if(num != 0 && num != 1)return false;
	//!STEP1 set port list
	m_port[num].pList = plist;
	//!STEP2 identyfy port by number
	return SetPortNumber(&m_port[num],PortNumber);
}

// Connect
bool RTCompConnect::Connect(std::string ConnectionName,std::string subs_type, std::string period)
{
	unsigned int i;
	//!³ʤɤΥݡȤȤɤΥݡȤ³Τ
  RTC::ConnectorProfile prof;
  prof.connector_id = "";
  prof.name = CORBA::string_dup(ConnectionName.c_str());

  prof.ports.length(2);
	for(i=0;i<2;i++)
	{
		prof.ports[i] = m_port[i].pList[ m_port[i].numOfPort ];
	}

	//!³ץѥƥ(Ūֳ֤ʤ)
  CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.interface_type", "CORBA_Any"));
  CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.dataflow_type", "Push"));
  CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.subscription_type", subs_type.c_str()));
  if (period != ""){
		CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.push_interval",		 period.c_str()));
	}

	//!³β(³ƤݡȤ)
	DisConnect(m_connect_id);

  //!³
  RTC::ReturnCode_t ret;
  ret = m_port[0].pList[m_port[0].numOfPort]->connect(prof);
  assert(ret == RTC::RTC_OK);

	//!³ɽ
	if(ret == RTC::RTC_OK)std::cout << "=== Connection Success ===" <<std::endl;
	else                  std::cout << "=== Connection Failure ===" <<std::endl;
  std::cout << "Connector ID: " << prof.connector_id << std::endl;
  NVUtil::dump(prof.properties);
	std::cout << "==========================" << std::endl;
	m_connect_id = prof.connector_id;
	return (ret == RTC::RTC_OK);
}

//Disconnect
bool RTCompConnect::DisConnect(
	std::string ConnectionName     //! Connection name
)
{
	if(m_connect_id=="")return true;
	m_port[0].pList[m_port[0].numOfPort]->disconnect(m_connect_id.c_str());
	m_connect_id = "";
	//ǽ٤
	return true;
}

// operator
RTCompConnect& RTCompConnect::operator=(const RTCompConnect& right)
{
	m_nameserver0 = right.m_nameserver0;
	m_nameserver1 = right.m_nameserver1;
	m_port[0]     = right.m_port[0];
	m_port[1]     = right.m_port[1];
	return *this;
}

//!STEP1 ³ݡȤ
void RTCompConnect::SetPortList  (RTCompSet *port, std::string &ComponentName, RTC::CorbaNaming *NameServer)
{
	port->con.setObject(NameServer->resolve(ComponentName.c_str()));
  port->pList = port->con->get_ports();
}

//!STEP2 ³ݡȤ(̾)
bool RTCompConnect::SetPortName  (RTCompSet *port, std::string &PortName)
{
	std::string oneOfPortName;
	port->numOfPort = -1;
	for(unsigned int i(0);i<port->pList->length();i++)
	{
		oneOfPortName = port->pList[i]->get_port_profile()->name;
		if( oneOfPortName == PortName )
		{
			port->numOfPort = i;
			return true;
		}
	}
	return false;
}

//!STEP2 port ³ݡȤֹǡ
bool RTCompConnect::SetPortNumber(RTCompSet *port, int &PortNumber)
{
	if(port->pList->length()<1)return false;
	else
	{
		port->numOfPort = PortNumber;
		return true;
	}
}

#endif
